<?php

namespace App\Http\Controllers\Customer;

use App\Http\Controllers\Controller;
use App\Http\Requests\Customer\AddCustomerRequest;

use App\Http\Requests\Customer\CustomerIdRequest;
use App\Http\Requests\Customer\CustomerRechargeMoneyRequest;
use App\Http\Requests\Customer\EmailRequest;
use App\Http\Requests\Customer\LoginCustomerRequest;
use App\Http\Requests\Customer\ResetPasswordRequest;
use App\Http\Requests\Customer\ShowCustomersRequest;
use App\Http\Requests\Item\ItemIdRequest;
use App\Http\Resources\Customer\LoginCustomerResource;
use App\Http\Resources\Customer\ShowCustomersResource;
use App\Services\ChangeCurrencyService;
use App\Interfaces\Repository;
use App\Mail\CodeVerificationMail;
use App\Mail\ResetPasswordMail;
use App\Models\Customer;
use App\Models\Favorite;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Validation\ValidationException;


class CustomerController extends Controller
{

       public function __construct(public Repository $repository, public ChangeCurrencyService $service)
    {
        $this->middleware('permission:customers_management', ['only' => ['Delete','ShowAll', 'ActiveOrNot', 'TraderOrNot', 'RechargeMoney']]);
    }

    public function EmailVerificationCode(AddCustomerRequest $request)
    {
        $arr = Arr::only($request->validated(), ['email']);
        $code = \rand(11111, 99999);
        Mail::to($arr['email'])->send(new CodeVerificationMail($code));
        return \SuccessData('Verification Code', ['verification_code' => $code]);
    }

    public function ResetPasswordCode(EmailRequest $request)
    {
        $arr = Arr::only($request->validated(), ['email']);
        $code = \rand(11111, 99999);
        Mail::to($arr['email'])->send(new ResetPasswordMail($code));
        return \SuccessData('Reset Password Code', ['reset_password_code' => $code]);
    }

    public function Create(AddCustomerRequest $request)
    {
        $arr = Arr::only($request->validated(), ['user_name', 'email', 'password', 'phone', 'fcm_token']);
        $this->repository->Create(Customer::class, $arr);
        return \Success('Customer Added Successfully');
    }

    public function Login(LoginCustomerRequest $request)
    {
        $arr = Arr::only($request->validated(), ['email', 'password', 'fcm_token']);
        $customer = Customer::where('email', $arr['email'])->first();
        if (!Hash::check($arr['password'], $customer->password)) {
            throw ValidationException::withMessages(['Password Not Correct']);
        }
        if (\array_key_exists('fcm_token', $arr)) {
            $this->repository->Update(Customer::class, $customer->id, ['fcm_token' => $arr['fcm_token']]);
        }
        $token = $customer->createToken('authToken', ['Customer'])->accessToken;
        $customer['token'] = $token;
        return \SuccessData('Customer Login Successfully', new LoginCustomerResource($customer));
    }


    public function ResetPassword(ResetPasswordRequest $request)
    {
        $arr = Arr::only($request->validated(), ['email', 'password']);
        $customer = Customer::where('email', $arr['email'])->first();
        $this->repository->Update(Customer::class, $customer->id, $arr);
        return \Success('Password Changes Successfully');
    }

    public function Logout()
    {
        $customer = \auth('Customer')->user();
        $customer->update(['fcm_token' => null]);
        $customer->tokens()->where('scopes', '["Customer"]')->delete();
        return \Success('Customer Logout Successfully');
    }

    public function ShowAll(ShowCustomersRequest $request)
    {
        $arr = Arr::only($request->validated(), ['search', 'isTrader']);
        $where = [];
        $perPage = \returnPerPage();

        if (\array_key_exists('isTrader', $arr)) {
            $where = \array_merge($where, ['is_trader' => $arr['isTrader']]);
        }
        $customers = $this->repository->ShowAll(Customer::class, $where);
        if (\array_key_exists('search', $arr)) {
            $customers = $customers->where('user_name', 'like', '%' . $arr['search'] . '%')
                ->orWhere('email', 'like', '%' . $arr['search'] . '%')
                ->orWhere('phone', 'like', '%' . $arr['search'] . '%');
        }
        $customers = $customers->paginate($perPage);
        ShowCustomersResource::collection($customers);
        return \Pagination($customers);
    }

    public function ActiveOrNot(CustomerIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['customerId']);
        $this->repository->ActiveOrNot(Customer::class, $arr['customerId']);
        return \Success('Customer Update Successfully');
    }

    public function TraderOrNot(CustomerIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['customerId']);
        $customer = $this->repository->ShowById(Customer::class, $arr['customerId']);
        $this->repository->Update(Customer::class, $arr['customerId'], ['is_trader' => !$customer->is_trader]);
        return \Success('Customer Update Successfully');
    }

    public function RechargeMoney(CustomerRechargeMoneyRequest $request)
    {
        $arr = Arr::only($request->validated(), ['customerId', 'quantity']);
        $customer = $this->repository->ShowById(Customer::class, $arr['customerId']);
        $customer->increment('balance', $arr['quantity']);
        return \Success('Recharge Money Successfully');
    }

    public function Delete(CustomerIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['customerId']);
        $this->repository->DeleteById(Customer::class, $arr['customerId']);
        return \Success('Customer Deleted Successfully');
    }
public function ShowBalance(CustomerIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['customerId']);
        $currency = \returnCurrency();
        $balanceCurrency = 'Dinar';
        $customer = $this->repository->ShowById(Customer::class, $arr['customerId']);
        $arr = [
            'balance' => (string) $this->service->ChangePriceByCurrency($customer->balance, $balanceCurrency, $currency),
            'currency' => $this->service->ChangeCurrency($currency),
        ];
        return \SuccessData('Balance Found Successfully', $arr);
    }
}
