<?php

namespace App\Http\Controllers\Item;

use App\Http\Controllers\Controller;
use App\Http\Requests\Item\AddItemRequest;
use App\Http\Requests\Item\ItemColorIdRequest;
use App\Http\Requests\Item\ItemIdRequest;
use App\Http\Requests\Item\ShowItemsRequest;
use App\Http\Requests\Item\UpdateItemRequest;
use App\Http\Resources\Item\ShowFavoriteItemsResource;
use App\Http\Resources\Item\ShowItemsResource;
use App\Http\Resources\Item\ShowOneItemResource;
use App\Interfaces\Repository;
use App\Models\Cart;
use App\Models\CartItem;
use App\Models\Currency;
use App\Models\Favorite;
use App\Models\Image;
use App\Models\Item;
use App\Models\ItemColor;
use App\Models\ItemOffer;

//
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\App;

class ItemController extends Controller
{
    public function __construct(public Repository $repository)
    {
        $this->middleware('permission:category_items_management', ['only' => ['Delete', 'Create', 'ActiveOrNot', 'SpecialOrNot', 'DeleteItemColor', 'Update']]);
    }

    public function Create(AddItemRequest $request)
    {
        $arr = Arr::only($request->validated(),
            ['category_id', 'brand_id', 'currency_id', 'storage_id', 'name_ar', 'name_en', 'description_ar', 'description_en', 'price', 'images', 'colors']);
        $item = $this->repository->Create(Item::class, $arr);
        $this->CreateItemImages($item->id, $arr['images']);
        if (\array_key_exists('colors', $arr)) {
            $this->CreateItemColors($item->id, $arr['colors']);
        }
        return \Success('Item Add Successfully');
    }

    private function CreateItemImages($id, array $images)
    {
        $path = 'Images/Items/';
        foreach ($images as $image) {
            Image::create([
                'imageable_type' => Item::class,
                'imageable_id' => $id,
                'url' => \uploadImage($image, $path),
            ]);
        }
    }

    private function CreateItemColors($id, array $colors)
    {
        $item = $this->repository->ShowById(Item::class, $id);
        $item->ItemColors()->delete();

        foreach ($colors as $color) {

            ItemColor::create([
                'item_id' => $id,
                'color_id' => $color,
            ]);

        }

    }

    public function ActiveOrNot(ItemIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['itemId']);
        $this->repository->ActiveOrNot(Item::class, $arr['itemId']);
        return \Success('Item Changed Successfully');
    }

    public function SpecialOrNot(ItemIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['itemId']);
        $item = $this->repository->ShowById(Item::class, $arr['itemId']);
        $this->repository->Update(Item::class, $arr['itemId'], ['is_special' => !$item->is_special]);
        return \Success('Item Changed Successfully');
    }

    public function ShowById(ItemIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['itemId']);
        $item = $this->repository->ShowById(Item::class, $arr['itemId']);
        return \SuccessData('Item Found Successfully', new ShowOneItemResource($item));
    }

    public function ShowAll(ShowItemsRequest $request)
    {
        $arr = Arr::only($request->validated(), ['search', 'categoryId', 'special', 'price', 'currency']);
        $where = [];
        $perPage = \returnPerPage();
        if (\array_key_exists('categoryId', $arr)) {
            $where = \array_merge($where, ['category_id' => $arr['categoryId']]);
        }
        if (\array_key_exists('search', $arr)) {
            $language = \returnLanguage();
            $where = \array_merge($where, [['name_' . $language, 'like', '%' . $arr['search'] . '%']]);
        }
        if (\array_key_exists('special', $arr)) {
            $where = \array_merge($where, ['is_special' => $arr['special']]);
        }
        if (\array_key_exists('currency', $arr)) {
            $currencyId = $this->repository->ShowAll(Currency::class, ['name' => $arr['currency']])->first()->id;
            $where = \array_merge($where, ['currency_id' => $currencyId]);
        }
        $items = $this->repository->ShowAll(Item::class, $where);

        if (\request()->route()->getPrefix() == 'customer_api') {
            $items->Active();
        }

//        if (\array_key_exists('price',$arr)) {
//            $currency = \returnCurrency();
//            $include = App::make('App\Services\ChangeCurrencyService');
//            foreach ($items->get() as  $item) {
//                $price = $include->ChangePriceByCurrency($arr['price'], $currency, $item->Currency->name);
//                $item_price = $item->ItemOffer()->Active()->first() ? $item->ItemOffer()->Active()->first()->offer_price : $item->price;
//                if ($item_price > $price ) {
//                    $items->where('id', '!=', $item->id);
//                }
//            }
//        }

        if (\array_key_exists('price', $arr)) {
            $items = $items->orderBy('price', $arr['price']);
        }
        $items = $items->paginate($perPage);
        ShowItemsResource::collection($items);
        return \Pagination($items);
    }

    public function Update(UpdateItemRequest $request)
    {
        $arr = Arr::only($request->validated(), ['itemId', 'category_id', 'brand_id', 'storage_id', 'name_ar', 'name_en', 'description_ar',
            'description_en', 'price', 'images', 'colors']);
        $this->repository->Update(Item::class, $arr['itemId'], $arr);
        if (\array_key_exists('images', $arr)) {
            $this->CreateItemImages($arr['itemId'], $arr['images']);
        }
        if (\array_key_exists('colors', $arr)) {
            $this->CreateItemColors($arr['itemId'], $arr['colors']);
        } else {
            $this->CreateItemColors($arr['itemId'], []);
        }
        return \Success('Item Update Successfully');
    }

    public function DeleteItemColor(ItemColorIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['itemId', 'colorId']);
        $this->repository->ShowAll(ItemColor::class, [
            'item_id' => $arr['itemId'],
            'color_id' => $arr['colorId'],
        ])->first()->delete();
        return \Success('Item Color Deleted Successfully');
    }

    public function LikeOrNot(ItemIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['itemId']);
        $where['item_id'] = $arr['itemId'];
        $where['customer_id'] = \auth('Customer')->user()->id;
        $favorite = $this->repository->ShowAll(Favorite::class, $where)->first();
        if ($favorite) {
            $favorite->delete();
        } else {
            $this->repository->Create(Favorite::class, $where);
        }
        return \Success('Like Update Successfully');
    }

    public function ShowFavoriteItems()
    {
        $customer_id = \auth('Customer')->user()->id;
        $perPage = \returnPerPage();
        $where = [
            'customer_id' => $customer_id,
        ];
        $favorites = $this->repository->ShowAll(Favorite::class, $where)->paginate($perPage);
        ShowFavoriteItemsResource::collection($favorites);
        return \Pagination($favorites);
    }

    public function Delete(ItemIdRequest $request)
    {
        $arr = Arr::only($request->validated(), ['itemId']);
        $this->repository->DeleteById(Item::class, $arr['itemId']);
        $this->repository->ShowAll(ItemColor::class, ['item_id' => $arr['itemId']])->delete();
        $this->repository->ShowAll(ItemOffer::class, ['item_id' => $arr['itemId']])->delete();
        $this->repository->ShowAll(Favorite::class, ['item_id' => $arr['itemId']])->delete();
        $item_id = $arr['itemId'];
        $carts = CartItem::where('item_id', $item_id)->whereHas('Cart', function ($query) {
            $query->where('status', 'initial');
        })->delete();
        return \Success('Item Deleted Successfully');
    }

}
